@@ -1,45 +1,4 @@
-//! [![congee](https://github.com/XiangpengHao/congee/actions/workflows/ci.yml/badge.svg)](https://github.com/XiangpengHao/congee/actions/workflows/ci.yml)
-//! [![Crates.io](https://img.shields.io/crates/v/congee.svg)](
-//! https://crates.io/crates/congee)
-//! [![dependency status](https://deps.rs/repo/github/xiangpenghao/congee/status.svg)](https://deps.rs/crate/congee)
-//! [![codecov](https://codecov.io/gh/XiangpengHao/congee/branch/main/graph/badge.svg?token=x0PSjQrqyR)](https://codecov.io/gh/XiangpengHao/congee)
-//! [![Documentation](https://docs.rs/congee/badge.svg)](https://docs.rs/congee)
-//!
-//! A Rust implementation of ART-OLC [concurrent adaptive radix tree](https://db.in.tum.de/~leis/papers/artsync.pdf).
-//! It implements the optimistic lock coupling with proper SIMD support.
-//!
-//! The code is based on its [C++ implementation](https://github.com/flode/ARTSynchronized), with many bug fixes.
-//!
-//! It only supports (and is optimized for) 8 byte key;
-//! due to this specialization, this ART has great performance -- basic operations are ~40% faster than [flurry](https://github.com/jonhoo/flurry) hash table, range scan is an order of magnitude faster.
-//!
-//! The code is extensively tested with [{address|leak} sanitizer](https://doc.rust-lang.org/beta/unstable-book/compiler-flags/sanitizer.html) as well as [libfuzzer](https://llvm.org/docs/LibFuzzer.html).
-//!
-//! ### Why Congee?
-//! - Fast performance, faster than most hash tables.
-//! - Concurrent, super scalable, it reaches 150Mop/s on 32 cores.
-//! - Super low memory consumption. Hash tables often have exponential bucket size growth, which often lead to low load factors. ART is more space efficient.
-//!
-//!
-//! ### Why not Congee?
-//! - Not for arbitrary key size. This library only supports 8 byte key.
-//!
-//! ### Example:
-//! ```
-//! use congee::Art;
-//! let art = Art::new();
-//! let guard = art.pin(); // enter an epoch
-//!
-//! art.insert(7, 42, &guard); // insert a value
-//! let val = art.get(&7, &guard).unwrap(); // read the value
-//! assert_eq!(val, 42);
-//!
-//! let mut scan_buffer = vec![(0, 0); 8];
-//! let scan_result = art.range(&0, &10, &mut scan_buffer, &guard); // scan values
-//! assert_eq!(scan_result, 1);
-//! assert_eq!(scan_buffer[0], (7, 42));
-//! ```
-
+#![doc = include_str!("../README.md")]
 #![allow(clippy::comparison_chain)]
 #![allow(clippy::len_without_is_empty)]
 
@@ -64,7 +23,7 @@ mod stats;
 #[cfg(test)]
 mod tests;
 
-use std::vec;
+use std::{mem::ManuallyDrop, vec};
 
 use base_node::BaseNode;
 pub use key::RawKey;
@@ -220,7 +179,7 @@ impl ArtUsize {
 
 /// The main adaptive radix tree.
 pub struct Art<V: Clone> {
-    inner: RawTree<UsizeKey>,
+    inner: ManuallyDrop<RawTree<UsizeKey>>,
     pt: std::marker::PhantomData<V>,
 }
 
@@ -268,7 +227,7 @@ impl<V: Clone> Art<V> {
     /// ```
     pub fn new() -> Self {
         Art {
-            inner: RawTree::new(),
+            inner: ManuallyDrop::new(RawTree::new()),
             pt: std::marker::PhantomData,
         }
     }
