@@ -4,11 +4,11 @@ use crossbeam_epoch::Guard;
 
 use crate::{
     base_node::{BaseNode, Node, Prefix},
-    child_ptr::NodePtr,
     key::RawKey,
     lock::ReadGuard,
     node_256::Node256,
     node_4::Node4,
+    node_ptr::NodePtr,
     range_scan::RangeScan,
     utils::{ArtError, Backoff},
 };
@@ -32,15 +32,18 @@ impl<K: RawKey> Default for RawTree<K> {
 impl<T: RawKey> Drop for RawTree<T> {
     fn drop(&mut self) {
         let v = unsafe { ManuallyDrop::take(&mut self.root) };
-        let mut sub_nodes = vec![Box::into_raw(v) as *const BaseNode];
+        let mut sub_nodes = vec![(Box::into_raw(v) as *const BaseNode, 0)];
 
         while !sub_nodes.is_empty() {
-            let node = sub_nodes.pop().unwrap();
+            let (node, level) = sub_nodes.pop().unwrap();
 
             let children = unsafe { &*node }.get_children(0, 255);
             for (_k, n) in children {
-                if !n.is_leaf() {
-                    sub_nodes.push(n.as_ptr());
+                if level != 7 {
+                    sub_nodes.push((
+                        n.as_ptr(),
+                        level + 1 + unsafe { &*n.as_ptr() }.prefix().len(),
+                    ));
                 }
             }
             unsafe {
@@ -89,13 +92,12 @@ impl<T: RawKey> RawTree<T> {
 
                 let child_node = child_node?;
 
-                if child_node.is_leaf() {
+                if level == 7 {
+                    // 7 is the last level, we can return the value
                     let tid = child_node.as_tid();
-                    if (level as usize) < key.len() - 1 {
-                        return None;
-                    }
                     return Some(tid);
                 }
+
                 level += 1;
 
                 node = if let Ok(n) = unsafe { &*child_node.as_ptr() }.read_lock() {
@@ -108,7 +110,7 @@ impl<T: RawKey> RawTree<T> {
     }
 
     #[inline]
-    fn insert_inner(&self, k: &T, tid: usize, guard: &Guard) -> Result<(), ArtError> {
+    fn insert_inner(&self, k: &T, tid: usize, guard: &Guard) -> Result<Option<usize>, ArtError> {
         let mut parent_node = None;
         let mut next_node = self.root.as_ref() as *const Node256 as *const BaseNode;
         let mut parent_key: u8;
@@ -136,7 +138,7 @@ impl<T: RawKey> RawTree<T> {
                         n
                     } else {
                         let new_leaf = {
-                            if level as usize == k.len() - 1 {
+                            if level == 7 {
                                 // last key, just insert the tid
                                 NodePtr::from_tid(tid)
                             } else {
@@ -157,7 +159,7 @@ impl<T: RawKey> RawTree<T> {
                             new_leaf,
                             guard,
                         ) {
-                            if level as usize != k.len() - 1 {
+                            if level != 7 {
                                 unsafe {
                                     // TODO: this is UB
                                     std::ptr::drop_in_place(new_leaf.as_ptr() as *mut BaseNode);
@@ -166,22 +168,22 @@ impl<T: RawKey> RawTree<T> {
                             return Err(e);
                         }
 
-                        return Ok(());
+                        return Ok(None);
                     };
 
                     if let Some(p) = parent_node {
                         p.unlock()?;
                     }
 
-                    if next_node_tmp.is_tid() {
+                    if level == 7 {
                         // At this point, the level must point to the last u8 of the key,
                         // meaning that we are updating an existing value.
                         let mut write_n = node.upgrade().map_err(|(_n, v)| v)?;
-                        write_n
+                        let old = write_n
                             .as_mut()
                             .change(k.as_bytes()[level as usize], NodePtr::from_tid(tid));
 
-                        return Ok(());
+                        return Ok(Some(old.as_tid()));
                     }
                     next_node = next_node_tmp.as_ptr();
                     level += 1;
@@ -199,7 +201,7 @@ impl<T: RawKey> RawTree<T> {
                     );
 
                     // 2)  add node and (tid, *k) as children
-                    if next_level as usize == k.len() - 1 {
+                    if next_level == 7 {
                         // this is the last key, just insert to node
                         new_node.insert(k.as_bytes()[next_level as usize], NodePtr::from_tid(tid));
                     } else {
@@ -228,7 +230,7 @@ impl<T: RawKey> RawTree<T> {
                     write_n
                         .as_mut()
                         .set_prefix(&prefix[0..(prefix_len - (next_level - level + 1) as usize)]);
-                    return Ok(());
+                    return Ok(None);
                 }
             }
             parent_node = Some(node);
@@ -236,10 +238,15 @@ impl<T: RawKey> RawTree<T> {
     }
 
     #[inline]
-    pub fn insert(&self, k: T, tid: usize, guard: &Guard) {
+    pub fn insert(&self, k: T, tid: usize, guard: &Guard) -> Option<usize> {
         let backoff = Backoff::new();
-        while self.insert_inner(&k, tid, guard).is_err() {
-            backoff.spin();
+        loop {
+            match self.insert_inner(&k, tid, guard) {
+                Ok(v) => return v,
+                Err(_e) => {
+                    backoff.spin();
+                }
+            }
         }
     }
 
@@ -359,12 +366,11 @@ impl<T: RawKey> RawTree<T> {
                         }
                     };
 
-                    if next_node_tmp.is_leaf() {
+                    if level == 7 {
                         key_tracker.push(node_key);
                         let full_key = key_tracker.to_usize_key();
-                        let input_key = std::intrinsics::bswap(unsafe {
-                            *(k.as_bytes().as_ptr() as *const usize)
-                        });
+                        let input_key =
+                            unsafe { *(k.as_bytes().as_ptr() as *const usize) }.swap_bytes();
                         if full_key != input_key {
                             return Ok(None);
                         }
