@@ -1,9 +1,4 @@
-use crate::base_node::NodeType;
-use crate::child_ptr::NodePtr;
-
-pub(crate) fn convert_type_to_version(n_type: NodeType) -> usize {
-    (n_type as usize) << 62
-}
+use crate::node_ptr::NodePtr;
 
 use core::cell::Cell;
 use core::fmt;
@@ -111,13 +106,13 @@ impl KeyTracker {
     pub(crate) fn to_usize_key(&self) -> usize {
         assert!(self.len == 8);
         let val = unsafe { *((&self.data) as *const [u8; 8] as *const usize) };
-        std::intrinsics::bswap(val)
+        val.swap_bytes()
     }
 
     #[inline]
     pub(crate) fn append_prefix(node: NodePtr, key_tracker: &KeyTracker) -> KeyTracker {
         let mut cur_key = key_tracker.clone();
-        if node.is_leaf() {
+        if key_tracker.len() == 8 {
             cur_key
         } else {
             let node_ref = unsafe { &*node.as_ptr() };
